-- JPM: This file is not imported from anywhere. It is also not listed in the
-- cabal package. Ideally it should be moved outside src/HarmTrace.

{-# OPTIONS_GHC -Wall -fno-warn-orphans #-}
{-# LANGUAGE ScopedTypeVariables #-}
module HarmTrace.Audio.Testing where

-- Music stuff
import Main (defaultOpts) 
import HarmTrace.Models.Pop.Instances ()
import HarmTrace.IO.Main (parseAnnotation)
import HarmTrace.Base.Parsing
import HarmTrace.Base.MusicRep
import HarmTrace.Models.Models
import HarmTrace.HarmTrace (ParseResult (..))

-- Audio stuff
import HarmTrace.Audio.Parser
import HarmTrace.Audio.Key
import HarmTrace.Audio.BeatChroma
-- import HarmTrace.Audio.Harmonize
import HarmTrace.Audio.Annotations
import HarmTrace.Audio.ChordTypes
import HarmTrace.Audio.Utils

-- Library modules
-- import Data.List (group)
-- import Data.Maybe (fromJust)
import System.FilePath
import Text.Printf(printf)
import Numeric.LinearAlgebra 

--------------------------------------------------------------------------------
-- Testing Audio stuff
--------------------------------------------------------------------------------

testDataPath, dataPath, gtPath, beatlesPath  :: FilePath
testDataPath = ".." </> "data" </> "audioTestData"
dataPath     = ".." </> "data" </> "AudioFeatures"
gtPath       = ".." </> "data" </> "ground-truth"
beatlesPath  = ".." </> "data" </> "BeatlesFeatures"

testChromaData, testBeatData, testKeyData :: IO ()
testChromaData = readFile (testDataPath </> "bothChroma.txt") >>=  
                 disp . toChromaMatrix . parseData parseChordinoData  
testBeatData   = do -- f <- readFile (testDataPath </> "beats.txt")
                    f <- readFile (testDataPath</> "02_-_Misery_vamp_beats.csv")
                    let (beat,err) = parseDataWithErrors parseBeatData f
                    putStrLn ("beat errors: " ++ show err)
                    print beat

testKeyData = 
  do dat <- readFile (gtPath </> "BeatlesKeys" </> "03_-_A_Hard_Day's_Night_04_-_I'm_Happy_Just_To_Dance_With_You.lab") 
     bts <- readFile (beatlesPath </> "03_-_A_Hard_Day's_Night_04_-_I'm_Happy_Just_To_Dance_With_You_vamp_qm-vamp-plugins_qm-tempotracker_beats.csv")
     chr <- readFile (beatlesPath </> "03_-_A_Hard_Day's_Night_04_-_I'm_Happy_Just_To_Dance_With_You_vamp_nnls-chroma_nnls-chroma_bothchroma.csv")
     let (cl,_kerr)   = parseDataWithErrors parseKeyAnnotationData dat 
         (beat,_berr) = parseDataWithErrors parseBeatData bts
         (cds,_ec) = parseDataWithErrors parseChordinoData chr
         -- (_chrm, beats, keys) = syncWithAnnKey cds beat cl
     print cl
     -- print beats
     -- print keys
                    

testKeySegments :: IO ()
testKeySegments = do
  ckey  <- readFile (dataPath </> "Queen" </> 
    -- "01_-_A Kind Of Magic_vamp_qm-vamp-plugins_qm-keydetector_keystrength.csv")
    "04_-_I Want It All_vamp_qm-vamp-plugins_qm-keydetector_keystrength.csv")
  bdat  <- readFile (dataPath </> "Queen" </> 
    -- "01_-_A Kind Of Magic_vamp_qm-vamp-plugins_qm-tempotracker_beats.csv")
    "04_-_I Want It All_vamp_qm-vamp-plugins_qm-tempotracker_beats.csv")
  cdat  <- readFile (dataPath </> "Queen" </> 
    -- "01_-_A Kind Of Magic_vamp_nnls-chroma_nnls-chroma_bothchroma.csv")
    "04_-_I Want It All_vamp_nnls-chroma_nnls-chroma_bothchroma.csv")
  let (key,_ek) = parseDataWithErrors parseKeyStrengthData ckey
      (cds,_ec) = parseDataWithErrors parseChordinoData cdat
      (bts,_eb) = parseDataWithErrors parseBeatData bdat
      -- kend   = getLastFrameTimeStamp key
      -- cend   = getLastFrameTimeStamp cds
      (chrm, beats, keys) = syncEndings cds bts key
      keygrp = mergeAndTimeStamp head beats . groupKeys
             $ toBeatSyncKey beats keys   
      chdgrp = mergeAndTimeStamp avgPC beats . groupPChord
             . createChordRanks $ beatSync beats chrm
      segs   = segmentByKey keygrp chdgrp
  putStrLn ("kend: " ++ show (last keys))
  putStrLn ("cend: " ++ show (last chrm))
  putStrLn ("btend: " ++ show (last beats))
  mapM_ print segs


testChromaKey :: IO ()
testChromaKey = do
  cdat  <- readFile (beatlesPath </> 
    -- "01_-_Please_Please_Me_01_-_I_Saw_Her_Standing_There_vamp_nnls-chroma_nnls-chroma_chroma.csv")
    -- "03_-_A_Hard_Day's_Night_04_-_I'm_Happy_Just_To_Dance_With_You_vamp_nnls-chroma_nnls-chroma_chroma.csv")
    -- "03_-_A_Hard_Day's_Night_11_-_When_I_Get_Home_vamp_nnls-chroma_nnls-chroma_chroma.csv")
    -- "04_-_Beatles_for_Sale_09_-_Words_of_Love_vamp_nnls-chroma_nnls-chroma_chroma.csv")
    "10CD1_-_The_Beatles_CD1_-_12_-_Piggies_vamp_nnls-chroma_nnls-chroma_chroma.csv")
  bdat  <- readFile (beatlesPath </>
    -- "01_-_Please_Please_Me_01_-_I_Saw_Her_Standing_There_vamp_qm-vamp-plugins_qm-tempotracker_beats.csv")
    -- "03_-_A_Hard_Day's_Night_04_-_I'm_Happy_Just_To_Dance_With_You_vamp_qm-vamp-plugins_qm-tempotracker_beats.csv")
    -- "03_-_A_Hard_Day's_Night_11_-_When_I_Get_Home_vamp_qm-vamp-plugins_qm-tempotracker_beats.csv")
    -- "04_-_Beatles_for_Sale_09_-_Words_of_Love_vamp_qm-vamp-plugins_qm-tempotracker_beats.csv")
    "10CD1_-_The_Beatles_CD1_-_12_-_Piggies_vamp_qm-vamp-plugins_qm-tempotracker_beats.csv")
  let (key,ek) = parseDataWithErrors parseChromaData cdat
      (bts,eb) = parseDataWithErrors parseBeatData bdat
  mapM_ print ek
  mapM_ print eb
  -- printKeyTable bts key
  print $ naiveBeatSyncKey bts key
  print $ getBeatSyncKeyFromChroma bts key
  
  -- printKeyTable bts key
  -- printKeyStrength bts key
  
disp :: Matrix NumData -> IO ()  
disp = putStr . format "  " (printf "%.2f")

testChordRanks :: IO()
testChordRanks  = do
  -- cdat  <- readFile (testDataPath </> "02_-_Misery_vamp_bothchroma.csv")
  -- bdat  <- readFile (testDataPath </> "02_-_Misery_vamp_beats.csv")
  cdat  <- readFile (dataPath </> "Queen" </> 
    "01_-_A Kind Of Magic_vamp_nnls-chroma_nnls-chroma_bothchroma.csv")
  bdat  <- readFile (dataPath </> "Queen" </> 
    "01_-_A Kind Of Magic_vamp_qm-vamp-plugins_qm-tempotracker_beats.csv")
  let crm = parseData parseChordinoData cdat
      bts = parseData parseBeatData bdat  
      -- end = getLastFrameTimeStamp crm
      -- ranks = groupBy probChordEq . createChordRanks $ beatSync bts crm
      ranks = mergeAndTimeStamp avgPC bts . groupPChord
            . createChordRanks $ beatSync bts crm
  mapM_ print ranks
  putStrLn ("length ranks: " ++ show (length ranks))

  
testAnnotation :: IO ()  
testAnnotation = readFile (gtPath </> "Queen" </> "01_ A_Kind_Of_Magic.lab") >>= 
                 print . parseData parseAnnotationData

testKeyAnnotation :: IO ()  
testKeyAnnotation = readFile (gtPath </> "Beatles_key" </> "07_-_Please_Please_Me.lab") >>= 
                    print . parseData parseKeyAnnotationData                                  
                 
testAnnotationParse :: IO ()
testAnnotationParse = putStrLn . show . pieceTreeHAn =<<
  parseAnnotation Jazz defaultOpts
                  (gtPath </> "Beatles_key" </> "07_-_Please_Please_Me.lab")
                  (gtPath </> "Beatles" </> "07_-_Please_Please_Me.lab") 
                  
 